      module smoothadjust
!****************************************************************
! This module is a component of VINE, copyright by the authors, *
! as listed below. Please see the COPYING and LICENSE files     *
! in the parent directory for additional information.           *
! This notification is present as an attempt to follow the      *
! recommendations for licensing published on the FSF page.      * 
!****************************************************************
!                                                               *
! This is a VINE module contains a routine for use in           *
! setting up initial conditions for simulations. It adjust      *
! smoothing lengths of SPH particles.                           *
!                                                               *
! Caveat Emptor: This routine is not very production            *
!    quality. You will need to examine things in it to          *
!    see if you need to modify them for your problem. Probably  *
!    you will.                                                  *
!                                                               *
! Routines:                                                     *
!   hadjust : adjust smoothing lengths                          *
!                                                               *
! Author: Andy Nelson 2008                                      *
!                                                               *
!****************************************************************
      use dimens
      use partvars
      use indivtsvars
      use clmpvars
      use miscvars
      use sphvars
      use treemake, only: rtree
      use tree, only: getparticles,initgetneigh,getneigh
      use constants
!
! Makes it possible to compile this without sph turned on at all
! (see stubs.F)
!
#ifdef SPH
      use sph, only: density
#endif

      implicit none

      private
      public hadjust


      contains
!------------------------------------------------------------------
      subroutine hadjust
!
! adjust h so that all particles have an appropriate
! number of neighbors
!
      integer moreneig,nstart,i,j,iboth,ipart
      integer iclump,jclump
      integer npsup,npund,maxnei,minnei,neimin_i,neimax_i,nbest
      double precision hminval,hmaxval,rat

      double precision distmas(ndim3,maxbite,maxclumpsize)
      double precision WdW(2,maxbite,maxclumpsize)
      integer nneigh(maxclumpsize),listnay(maxbite,maxclumpsize)
      integer ntot_a,listpt_a(maxclumpsize)
      integer ntot_i,listpt_i(maxclumpsize)

      character(len=20), parameter::where='hadjust'
      if(idbg.gt.3)print *,where

#if NDIM==2
      neimin_i=17
      neimax_i=23
#else 
      neimin_i=55
      neimax_i=65
#endif
      nbest=(neimax_i+neimin_i)/2

      npund=1
      npsup=1
      do while(npsup+npund.gt.0)
         call rtree(igrav,imac,0,igrape,ifullextrap,irevise,npart,
     &              nactive,iactive,nactive_s,iactive_s,eps,treeacc,
     &              boxlims,posmas,h,numclumps,lstclumps,hclmp_is_exact)
         npund=0
         npsup=0

         maxnei=0
         minnei=npart+1

         call initgetneigh
         do jclump=1,nactive_c
            iclump=iactive_c(jclump)
            call getparticles(iclump,maxclumpsize,
     &                        ntot_a,ntot_i,listpt_a,listpt_i)
            do j=1,ntot_a
               i=listpt_a(j)
               nneightot(i)=0
            enddo

            moreneig=1
            nstart=0
            iboth=0
            do while(moreneig.ne.0)
               call getneigh(ibctype,iboth,iclump,ntot_a,ntot_i,
     &                listpt_a,listpt_i,nstart,
     &                nneigh,listnay,distmas,moreneig)
               do j=1,ntot_a
                  i=listpt_a(j)
                  nneightot(i)=nneightot(i) + nneigh(j)
               enddo
            enddo
            do j=1,ntot_a
               i=listpt_a(j)
               if(nneightot(i).lt.neimin_i)then
                  h(i)=half*h(i)*(one + 
     &                  (dble(nbest)/dble(max(1,nneightot(i))))**diminv)
                  npund=npund + 1
               endif
               if(nneightot(i).gt.neimax_i)then
                  h(i)=half*h(i)*(one + 
     &                  (dble(nbest)/dble(max(1,nneightot(i))))**diminv)
                  npsup=npsup + 1
               endif
               minnei=min(minnei,nneightot(i))
               maxnei=max(maxnei,nneightot(i))
            enddo
         enddo
         write(*,*)' '
         write(*,30)npsup,maxnei
         write(*,31)npund,minnei
      enddo

      hmaxval=ZERO
      hminval=huge(ONE)
      do ipart=1,npart
         hmaxval=max(hmaxval,h(ipart))
         hminval=min(hminval,h(ipart))
      enddo
      print *,'hmaxval:',hmaxval
      print *,'hminval:',hminval

11    format(i7,10(1pd15.7))

30    format('number above limit: ',i9,' max naybs: ',i9)
31    format('number below limit: ',i9,' min naybs: ',i9)
      return
      end subroutine hadjust
!------------------------------------------------------------------
      end module smoothadjust
