/*
 * header for GRAPE-5 user library
 */

#ifndef GP5UTIL_H
#define GP5UTIL_H

/* constant definitions */

#define NCHIPS (8) /* # of G5 chips per board */
#define NRP (2) /* # of real pipelines per chip */
#define NVP (12) /* # of virtual pipelines per chip */
#define NPIPES (NCHIPS*NVP)
#define JMEMSIZE (131072-1)
#define NBMEMSIZE (32768)
#define PIOWRITE(addr, value){*(((unsigned int *)mp)+(addr)) = (value);}
#define DEFAULT_FORCE_CORRECTION (1.0/1.0009)
#define DEFAULT_POTENTIAL_CORRECTION (1.0/1.0006)

#if !defined(__LANGUAGE_FORTRAN__) && !defined(_LANGUAGE_FORTRAN)

#if defined(__cplusplus)
extern "C"
{
#endif

/* C interface */

/*
 * standard functions
 * (the number of the boards is hidden to the user)
 */
void g5_open(void);
void g5_open_noinit(void);
void g5_close(void);
int g5_get_number_of_pipelines(void);
void g5_set_range(double xmin, double xmax, double mmin);
void g5_get_range(double *xmin, double *xmax, double *mmin);
void g5_set_n(int n);
void g5_set_cell_index_table(int nc, int *j0, int *nj);
void g5_set_mj(int adr, int nj, double *mj);
void g5_set_mj_safe(int adr, int nj, double *mj);
void g5_set_xj(int adr, int nj, double (*xj)[3]);
void g5_set_xj_safe(int adr, int nj, double (*xj)[3]);
void g5_set_xmj(int adr, int nj, double (*xj)[3], double *mj);
void g5_set_eps_to_all(double eps);
void g5_set_eps(int ni, double *eps);
void g5_set_h_to_all(double h);
void g5_set_h(int ni, double *h);
void g5_set_xi(int ni, double (*xi)[3]);
void g5_set_ip(int ni, double (*xi)[3], double *eps, double *h);
void g5_run(void);
void g5_get_force(int ni, double (*a)[3], double *p);
void g5_calculate_force_on_x(double (*x)[3], double (*a)[3], double *p, int ni);


/* necessary only when you need neighbor particles */
int g5_read_neighbor_list(void);
int g5_get_neighbor_list(int ip, int *list);

/* necessary only when you calculate force with cutoff */
void g5_set_cutoff_table(double (*ffunc)(double), double fcut, double fcor,
                        double (*pfunc)(double), double pcut, double pcor);
void g5_set_eta(double eta);
void g5_set_ishift(int ishift);

/* necessary only when you use cell-index method */
void g5_set_cell_index_table(int nc, int *j0, int *nj);

/* necessary only when you need to do something on the host computer
   during GRAPE-5 calculationg the force */
int g5_is_busy(void);

/*
 * functions to control multiple boards individually
 * (the user need to specifies the board ID (devid) explicitly)
 */
void g5_openMC(int devid);
void g5_open_noinitMC(int devid);
void g5_closeMC(int devid);
int g5_get_number_of_pipelines_per_board(void);
int g5_get_firstcluster(void);
int g5_get_number_of_clusters(void);
int g5_get_number_of_boards(void);
void g5_set_rangeMC(int devid, double xmin, double xmax, double mmin);
void g5_get_rangeMC(int devid, double *xmin, double *xmax, double *mmin);
void g5_set_nMC(int devid, int n);
void g5_set_cell_index_tableMC(int devid, int nc, int *j0, int *nj);
void g5_set_mjMC(int devid, int adr, int nj, double *mj);
void g5_set_xjMC(int devid, int adr, int nj, double (*xj)[3]);
void g5_set_eps_to_allMC(int devid, double eps);
void g5_set_epsMC(int devid, int ni, double *eps);
void g5_set_h_to_allMC(int devid, double h);
void g5_set_hMC(int devid, int ni, double *h);
void g5_set_xiMC(int devid, int ni, double (*xi)[3]);
void g5_set_ipMC(int devid, int ni, double (*xi)[3], double *eps, double *h);
void g5_runMC(int devid);
void g5_get_forceMC(int devid, int ni, double (*a)[3], double *p);
int g5_read_neighbor_listMC(int devid);
int g5_get_neighbor_listMC(int devid, int ip, int *list);
void g5_set_cutoff_tableMC(int devid,
                          double (*ffunc)(double), double fcut, double fcor,
                          double (*pfunc)(double), double pcut, double pcor);
void g5_set_etaMC(int devid, double eta);
void g5_set_ishiftMC(int devid, int ishift);
int g5_is_busyMC(int devid);


/* FORTRAN interface */

void g5_accel_(int *ni, double (*xi)[3], 
               int *nj, double (*xj)[3], double *mj,
               double (*a)[3], double *p,
               double *eps);

/* for administrator use only */
void g5_optimize_communication(int droff, double drratio, double dwratio);

#if defined(__cplusplus)
}
#endif

#endif /* LANGUAGE_FORTRAN */

#endif /* GP5UTIL_H */

