module perform
!****************************************************************
! This module is a component of VINE, copyright by the authors, *
! as listed below. Please see the COPYING and LICENSE files     *
! in the parent directory for additional information.           *
! This notification is present as an attempt to follow the      *
! recommendations for licensing published on the FSF page.      * 
!*********************************************************************
!  This file contains the VINE module to collect various             *
!  performance analysis information for the code.                    *
!                                                                    *
!  Written by Andy Nelson                                            *
!*********************************************************************
implicit none

public
private ik8
save

integer, parameter::ik8=selected_int_kind(18)

integer, parameter::ntimings=70

character(len=14), parameter::perfnames(ntimings) = (/    'Total time    ','Avg Dump Time ',                                   &
       'Total gravity ','  grav. accel ','  mtree       ','  rtree+reset ','  PM gravity  ','  reordering  ','  activate    ', &
       '  findmult    ','  Total grape ','    grape-IO  ','    grape-calc','    host-calc ','  Total ewald ','    FFT       ', &
       '    FFTW plan ','unusedTree    ','unusedTree    ','unusedTree    ','unusedTree    ','unusedTree    ','unusedTree    ', &
       'Total SPH     ','  density     ','  sphaccel    ','  eos         ','  udot        ','  alphadot    ','  hdot        ', &
       '  table build ','  sedsynth    ','unusedSPH     ','unusedSPH     ','unusedSPH     ','unusedSPH     ','unusedSPH     ', &
       'Total Integ   ','  halfupdate  ','  fullupdate  ','  error       ','  correction  ','  copyback    ','unusedInteg   ', &
       'Total indivTS ','  getactive   ','  getnext     ','  setnewdt    ','  updtstep    ','  updttime    ','unusedIndTS   ', &
       'Total IO calls','  rdump       ','  wdump       ','  PM IO       ','unusedIO      ','unusedIO      ','unusedIO      ', &
       'Total PM      ','  remove      ','unusedPM      ','unusedPM      ','unusedPM      ','unusedPM      ','unusedPM      ', &
       'Total Analysis','unused        ','unused        ','unused        ','unused        '                                    /)

integer, parameter ::                                     itme_tottime =1 ,itme_dumptime=2 ,                                   &
       itme_gravity =3 ,itme_gravacc =4 ,itme_mtree   =5 ,itme_rtree   =6 ,itme_PMgrav  =7 ,itme_reorder =8 ,itme_activate=9 , &
       itme_findmult=10,itme_grape   =11,itme_grpIO   =12,itme_grpcalc =13,itme_hostcalc=14,itme_ewald   =15,itme_fft     =16, &
       itme_fftwplan=17,itme_unusedT1=18,itme_unusedT2=19,itme_unusedT3=20,itme_unusedT4=21,itme_unusedT5=22,itme_unusedT6=23, &
       itme_SPH     =24,itme_density =25,itme_sphaccel=26,itme_eos     =27,itme_udot    =28,itme_alphadot=29,itme_hdot    =30, &
       itme_mktable =31,itme_sedsynth=32,itme_unusedS2=33,itme_unusedS3=34,itme_unusedS4=35,itme_unusedS5=36,itme_unusedS6=37, &
       itme_intgratr=38,itme_halfupdt=39,itme_fullupdt=40,itme_error   =41,itme_correct =42,itme_copyback=43,itme_unusedI1=44, &
       itme_indts   =45,itme_getactiv=46,itme_getnext =47,itme_setnewdt=48,itme_updtstep=49,itme_updttime=50,itme_unusdTS1=51, &
       itme_iocalls =52,itme_rdump   =53,itme_wdump   =54,itme_pmio    =55,itme_unusdIO1=56,itme_unusdIO2=57,itme_unusdIO3=58, &
       itme_PM      =59,itme_remove  =60,itme_unusdPM1=61,itme_unusdPM2=62,itme_unusdPM3=63,itme_unusdPM4=64,itme_unusdPM5=65, &
       itme_analysis=66

      double precision:: timing(ntimings) =                            &
                 (/ 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0,  &
                    0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0,  &
                    0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0,  &
                    0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0,  &
                    0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0,  &
                    0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0,  &
                    0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0 /)
      integer::  ncalls(ntimings) =                &
                 (/ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  &
                    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  &
                    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  &
                    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  &
                    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  &
                    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  &
                    0, 0, 0, 0, 0, 0, 0, 0, 0, 0 /)
!
! 1=sph, 2=nbody, 3=PM
!
integer(ik8)::  nactivetot(3) =(/0, 0, 0/)
integer(ik8):: nactiveftot(3) =(/0, 0, 0/)

double precision cput_start
double precision xcpus_inv


      contains
!-----------------------------------------------------------------------
subroutine initperf

integer numcpus
!
! IBM xlf has different possibilities for collecting time. This option
! forces the collection of all cpu time from all cpus. Then we
! have to correct for this to get the cpu time per cpu, which is what
! we want (essentially wall clock time). This statement doesn't 
! appear to be applicable to xlf running on linux. It isn't clear why,
! but on these systems, the time is already adjusted, so we don't have 
! to. 
! Make sure to define ibm in a preprocessor statement (i.e. -Dibm)
!
      xcpus_inv=1d0
#ifdef ibm
      call setrteopts('cpu_time_type=total_alltime')
#ifndef ibm5l
      call numprocessors(numcpus)
      xcpus_inv=1d0/dble(numcpus)
#endif
#endif
!
! Intel compiler v10 needs this. Intel v9 does not. Go figure.
!
#if (__INTEL_COMPILER>=1000)
      call numprocessors(numcpus)
      xcpus_inv=1d0/dble(numcpus)
#endif


      nactivetot=0
      nactiveftot=0
      timing=0d0
      ncalls=0

      call cpu_time(cput_start)

return
end subroutine initperf
!-----------------------------------------------------------------------
subroutine perfcollect(itme,where,startstop,ifamily,iroutine)
!
! Start using this routine some day soon, or more advanced
! version that has the same sort of behavior as current
! implementation in each timed routine.
!
character(len=*), intent(in):: where,startstop
integer, intent(in):: itme
integer, intent(in):: ifamily,iroutine

double precision, save:: tme1=0d0, tme2=0d0

      select case(startstop)
      case('start')
         call cpu_time(tme1)

      case('stop')
         call cpu_time(tme2)

         timing(iroutine)=timing(iroutine) + (tme2-tme1)
         ncalls(iroutine)=ncalls(iroutine) + 1

         timing(ifamily) =timing(ifamily)  + (tme2-tme1)
         ncalls(ifamily) =ncalls(ifamily)  + 1

         if(itme.eq.2)print *,where,xcpus_inv*(tme2-tme1)

      case default
         print *,'unknown startstop timing option in perfcollect'
         print *,'Will not collect time'
      end select 

return
end subroutine perfcollect
!-----------------------------------------------------------------------
subroutine perfsummary(npart)

integer, intent(in)::npart

integer(ik8) ntotcalcs,ntotfcalcs,npartlong
integer(ik8) nclong,npercall,nfpercall
integer i,numcpus
double precision tother,tbuild,cput_now
character(len=79) headerline

      call cpu_time(cput_now)
      call numprocessors(numcpus)

      timing(itme_tottime)=(cput_now-cput_start)
      ncalls(itme_tottime)=1

      ntotcalcs =nactivetot (1) + nactivetot (2) !don't count PMs alone
      ntotfcalcs=nactiveftot(1) + nactiveftot(2) !don't count PMs alone
      npartlong=npart
      nclong=max(1,ncalls(itme_getactiv))
      npercall=ntotcalcs/nclong
      nfpercall=ntotfcalcs/nclong

      write(*,*)'========================================================================'
      write(*,*)' '
      write(*,*)'                 Performance of the code so far'
      write(*,*)' '
      write(*,*)'========================================================================'
      write(*,*)' '
      write(*,30)xcpus_inv*timing(itme_tottime)/3600d0
      write(*,*)' '
      write(*,32)nactivetot(1)
      write(*,33)nactivetot(2)
      write(*,34)nactivetot(3)
      write(*,*)' '
      write(*,31)numcpus
      write(*,*)' '
      write(*,35)dble(npercall )/dble(npartlong)
      write(*,36)dble(nfpercall)/dble(npartlong)
      write(*,*)'(can be >1 if particle removal is turned on)'
      write(*,*)' '

      tother=timing(itme_tottime) - timing(itme_gravity ) - timing(itme_SPH  ) - timing(itme_PM      )   &
                                  - timing(itme_intgratr) - timing(itme_indts) - timing(itme_iocalls )   &
                                  - timing(itme_analysis)

      headerline='==============================================' 
      write(*,*)headerline
      headerline='Aggregate     |     Particles/s  |    Percent ' 
      write(*,*)headerline
      headerline='==============================================' 
      write(*,*)headerline

      print 37,perfnames(itme_tottime    ),    &
            dble(ntotcalcs )/max(1d-3,xcpus_inv*timing(itme_tottime )),     100d0*timing(itme_tottime )/timing(itme_tottime)

      if(timing(itme_gravity).gt.0d0)then
         print 37,perfnames(itme_gravity ),    &
            dble(ntotcalcs )/(xcpus_inv*timing(itme_gravity )),             100d0*timing(itme_gravity )/timing(itme_tottime)
      endif

      if(timing(itme_gravacc).gt.0d0)then
         print 37,perfnames(itme_gravacc ),    &
            dble(ntotcalcs )/(xcpus_inv*timing(itme_gravacc )),             100d0*timing(itme_gravacc )/timing(itme_tottime)
      endif

      if(timing(itme_PMgrav).gt.0d0)then
         print 37,perfnames(itme_PMgrav  ),    &
            dble(ntotcalcs )/(xcpus_inv*timing(itme_PMgrav  )),             100d0*timing(itme_PMgrav  )/timing(itme_tottime)
      endif

      tbuild=timing(itme_mtree)+timing(itme_rtree)
      if(tbuild.gt.0d0)then
         print 37,'  mrtree (eff)',            &
            dble(ntotcalcs)/(xcpus_inv*tbuild),                             100d0*tbuild/timing(itme_tottime)
      endif

      if(timing(itme_mtree).gt.0d0)then
         print 37,perfnames(itme_mtree   ),    &
            dble(npart*ncalls(itme_mtree))/(xcpus_inv*timing(itme_mtree)),  100d0*timing(itme_mtree   )/timing(itme_tottime)
      endif

      if(timing(itme_rtree).gt.0d0)then
         print 37,perfnames(itme_rtree   ),    &
            dble(npart*ncalls(itme_rtree))/(xcpus_inv*timing(itme_rtree)),  100d0*timing(itme_rtree   )/timing(itme_tottime)
      endif

      if(timing(itme_SPH).gt.0d0)then
         print 37,perfnames(itme_SPH     ),    &
            dble(nactivetot(1))/(xcpus_inv*timing(itme_SPH     )),          100d0*timing(itme_SPH     )/timing(itme_tottime)
      endif

      if(timing(itme_PM).gt.0d0)then
         print 37,perfnames(itme_PM      ),    &
            dble(nactivetot(1))/(xcpus_inv*timing(itme_PM      )),          100d0*timing(itme_PM      )/timing(itme_tottime)
      endif

      if(timing(itme_intgratr).gt.0d0)then
         print 37,perfnames(itme_intgratr),    &
            dble(ntotcalcs    )/(xcpus_inv*timing(itme_intgratr)),          100d0*timing(itme_intgratr)/timing(itme_tottime)
      endif

      if(timing(itme_indts   ).gt.0d0)then
         print 37,perfnames(itme_indts   ),    &
            dble(ntotcalcs    )/(xcpus_inv*timing(itme_indts   )),          100d0*timing(itme_indts   )/timing(itme_tottime)
      endif

      if(timing(itme_iocalls ).gt.0d0)then
         print 37,perfnames(itme_iocalls ),    &
            dble(ntotcalcs    )/(xcpus_inv*timing(itme_iocalls )),          100d0*timing(itme_iocalls )/timing(itme_tottime)
      endif

      if(timing(itme_analysis).gt.0d0)then
         print 37,perfnames(itme_analysis),    &
            dble(ntotcalcs    )/(xcpus_inv*timing(itme_analysis)),          100d0*timing(itme_analysis)/timing(itme_tottime)
      endif

      print 37,'Total other     ',             &
            dble(ntotcalcs    )/max(1d-3,xcpus_inv*tother),                 100d0*tother/timing(itme_tottime)

      write(*,*)' '

      write(*,*)'=========================================================================='
      headerline = 'Routine/group' // '|     Total Time  ' // '|    Per call    ' // '| N calls' // '|Percent of Total'
      write(*,*)headerline
      write(*,*)'=========================================================================='
      do i=1,ntimings
         if(timing(i).gt.1d-20)then
            write(*,40)perfnames(i),xcpus_inv*timing(i),xcpus_inv*timing(i)/dble(max(1,ncalls(i))),ncalls(i),  &
                                                                           100d0*timing(i)/timing(itme_tottime)
         endif
      enddo
      write(*,40)'Total other    ',tother,tother,1,100d0*tother/max(1d-3,xcpus_inv*timing(itme_tottime))
      write(*,*)'=========================================================================='
      write(*,*)' '


30    format('Total wall clock time        :',f12.4,' hr ')
31    format('Number of processors: ',i8)
32    format('Cumulative SPH   particles active :',i18)
33    format('Cumulative Nbody particles active :',i18)
34    format('Cumulative PM    particles active :',i18)
35    format('Fraction active per deriv call   :',f10.4)
36    format('Fraction finishing per deriv call:',f10.4,' (RK only)')
37    format(a14,' | ',f16.2,' | ',f10.2)
40    format(a14,'|',f14.2,' s |',f13.5,' s |',i8,'| ',f14.10)
end subroutine perfsummary
!------------------------------------------------------------------------------------
end module perform 
