module splines
!
! Module that contains two routines to calculate linear or cubic
! spline coefficients given data y, and spacing dx. Returns
! coefficients a
!
!
implicit none

public cubic, linear

contains
subroutine cubic(y,dx,a)
!**********************************************************************
!                                                                     *
!  subroutine to fit a cubic polynomial to 4 equally spaced data      *
!                                                                     *
!**********************************************************************

double precision, intent(in ):: y(4),dx(2,3)
double precision, intent(out):: a(4)
double precision, parameter:: sixth=1d0/6d0

      a(4)=y(2)
      a(2)=(0.5d0*(y(1)+y(3))-a(4))*dx(2,2)
      a(1)=(y(4)-3.d0*y(3)-y(1)+3.d0*a(4))*sixth*dx(2,3)
      a(3)=(y(3)-a(1)*dx(1,3)-a(2)*dx(1,2)-a(4))*dx(2,1)

return
end subroutine cubic
subroutine linear(y,dxi,a)
!**********************************************************************
!                                                                     *
!  subroutine to fit a linear polynomial to 2 data                    *
!                                                                     *
!**********************************************************************

double precision, intent(in ):: y(2),dxi
double precision, intent(out):: a(2)

      a(2)=y(1)
      a(1)=(y(2)-y(1))*dxi

return
end subroutine linear
end module splines
